(*
 *  beware IRC services, bwinnt.pas
 *  Copyright (C) 2002 Bas Steendijk
 *
 *  some modifications by peter green
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *)

{
code for NT system service
}

unit bwinnt;

interface

uses windows,
{$ifdef FPC}
  wsocket;  
{$else}
  messages,
  winsvc;
{$endif}



const
  outputfile='output.txt';

Const { from WINNT.H }
  Service_Win32_Own_Process   = $10;
  Service_Win32_Share_Process = $20;
  runasservice:boolean = false;

  SERVICE_DEMAND_START=$00000003;
  SERVICE_ERROR_NORMAL=$00000001;
  EVENTLOG_ERROR_TYPE=$0001;

Var
  StatusHandle : Service_Status_Handle;

procedure InstallService;
procedure unInstallService;
function runservice:boolean;

procedure reportstopped;
procedure reportstopping;

implementation

uses bconfig,bservunit;

procedure reportstopping;
Var
  Status : TServiceStatus;
begin
  if not runasservice then exit;
  With Status do Begin
    FillChar(Status,SizeOf(Status),0);
    dwServiceType := Service_Win32_Own_Process;
    dwCurrentState := Service_Stop_pending;
    dwControlsAccepted := Service_Accept_Stop;
  End;
  {$ifdef fpc}
    SetServiceStatus(StatusHandle,@Status);
  {$else}
    SetServiceStatus(StatusHandle,Status);
  {$endif}
end;

procedure reportstopped;
Var
  Status : TServiceStatus;
begin
  if not runasservice then exit;
  With Status do Begin
    FillChar(Status,SizeOf(Status),0);
    dwServiceType := Service_Win32_Own_Process;
    dwCurrentState := Service_Stopped;
    dwControlsAccepted := Service_Accept_Stop;
  End;
  {$ifdef fpc}
    SetServiceStatus(StatusHandle,@Status);
  {$else}
    SetServiceStatus(StatusHandle,Status);
  {$endif}
  processmessages;
end;

Procedure MyCtrlHandler(OpCode : Integer); StdCall;
Var
  Status : TServiceStatus;
Begin
  Case OpCode of
    Service_Control_Stop        : Begin
                                    shutdownreason := 'Stopped by NT services manager';
                                    globalshutdown;
                                  End;
    Service_Control_Interrogate : Begin { report our current status }
                                    With Status do Begin
                                      FillChar(Status,SizeOf(Status),0);
                                      dwServiceType := Service_Win32_Own_Process;
                                      dwCurrentState := Service_Running;
                                      dwControlsAccepted := Service_Accept_Stop;
                                    End;
                                    SetServiceStatus(StatusHandle,{$ifdef fpc}@Status{$else}Status{$endif});
                                  End;

    End;
End;


procedure servicemain(ArgCount : Integer; Args : Pointer); StdCall;
var
  Status : TServiceStatus;
begin
  StatusHandle := RegisterServiceCtrlHandler(@opt.servicename[1],@MyCtrlHandler);
  if statushandle = 0 then exit;
  runasservice := true;
  { Report status to Service Control Manager (SCM) }
  With Status do Begin
    FillChar(Status,SizeOf(Status),0);
    dwServiceType := Service_Win32_Own_Process;
    dwCurrentState := Service_Running;
    dwControlsAccepted := Service_Accept_Stop;
  End;
  SetServiceStatus(StatusHandle,{$ifdef fpc}@Status{$else}Status{$endif});

{!!!  getparams;}

  initapplication;
  runapplication;
  CleanupApplication;
end;


procedure InstallService;
var
   schService:SC_HANDLE;
   schSCManager:SC_HANDLE;
   lpszPath:array[0..511] of char;
   t:textfile;
begin
     assignfile(t,outputfile);
     rewrite(t);

     if GetModuleFileName(0,@lpszPath,sizeof(lpszPath))=0 then exit;

     schSCManager:=OpenSCManager(nil,nil,SC_MANAGER_ALL_ACCESS);
     if (schSCManager>0) then
     begin
          schService:=CreateService(schSCManager,@opt.ServiceName[1],@opt.ServiceName[1],
          SERVICE_ALL_ACCESS,SERVICE_WIN32_OWN_PROCESS,SERVICE_DEMAND_START,
          SERVICE_ERROR_NORMAL,@lpszPath,nil,nil,nil,nil,nil);
          if (schService>0) then
          begin
               Writeln(t,'Install Ok.');
               CloseServiceHandle(schService);
          end
          else
               Writeln(t,'Unable to install '+opt.ServiceName+', CreateService Fail. (service is already installed)');
     end
     else
         Writeln(t,'Unable to install '+opt.ServiceName+', OpenSCManager Fail.');
    closefile(t);
end;


procedure UnInstallService;
var
  ServiceControlHandle: SC_HANDLE;
  SCManagerHandle: SC_HANDLE;
  s:string;
  Status : TServiceStatus;
  t:textfile;
begin
  assignfile(t,outputfile);
  rewrite(t);

  s := opt.servicename;

  SCManagerHandle := OpenSCManager(nil, nil, SC_MANAGER_ALL_ACCESS);
  if (SCManagerHandle > 0) then begin
    ServiceControlHandle := OpenService(SCManagerHandle, @s[1], SERVICE_ALL_ACCESS);
    if (ServiceControlHandle > 0) then begin
      if ControlService(ServiceControlHandle, SERVICE_CONTROL_STOP, {$ifdef fpc}@Status{$else}Status{$endif}) then begin
        Write(t,'Stopping Service ');
        Sleep(1000);
        while (QueryServiceStatus(ServiceControlHandle, {$ifdef fpc}@Status{$else}Status{$endif})) do begin
          if Status.dwCurrentState = SERVICE_STOP_PENDING then begin
            Write(t,'.');
            Sleep(1000);
          end else
            break;
        end;
        WriteLn(t);

        if Status.dwCurrentState = SERVICE_STOPPED then
          writeLn(t,'Service Stop Now')
        else begin
          CloseServiceHandle(ServiceControlHandle);
          CloseServiceHandle(SCManagerHandle);

          WriteLn(t,'Service Stop Fail');
          closefile(t);
          exit;
        end;
      end;
      if (DeleteService(ServiceControlHandle)) then
        WriteLn(t,'Service Uninstall Ok.')
      else
        WriteLn(t,'DeleteService fail');
      CloseServiceHandle(ServiceControlHandle);
    end else
      WriteLn(t,'OpenService fail (service is not installed)');
    CloseServiceHandle(SCManagerHandle);
  end else
    WriteLn(t,'OpenSCManager fail');
    closefile(t);
end;

Var
  STEs : Array[0..1] of TServiceTableEntry;

function runservice:boolean;
begin
  With STEs[0] do Begin
    lpServiceName := @opt.servicename[1];
    lpServiceProc := @servicemain;
  End;
  With STEs[1] do Begin
    lpServiceName := nil;
    lpServiceProc := nil;
  End;
  runasservice := false;
  {$ifdef fpc}
    StartServiceCtrlDispatcher(@STEs[0]);
  {$else}
    StartServiceCtrlDispatcher(STEs[0]);
  {$endif}
  result := runasservice;
end;


end.
